import Client from "mtproton/envs/browser";
import makeMTProto from "mtproton/src/index";
import SHA1 from "mtproton/envs/browser/sha1";
import SHA256 from "mtproton/envs/browser/sha256";
import PBKDF2 from "mtproton/envs/browser/pbkdf2";
import Transport from "mtproton/envs/browser/transport";
import getRandomBytes from "mtproton/envs/browser/get-random-bytes";
import getLocalStorage from "mtproton/envs/browser/get-local-storage";

class TelegramHelper {
  constructor(appID, appHash, custom = false) {
    if (!custom) {
      this.client = new Client({
        api_id: appID,
        api_hash: appHash,
        test: false,
      });
    } else {
      let createTransport = function (dc, crypto) {
        return new Transport(dc, crypto);
      };

      const MTProto = makeMTProto({
        SHA1,
        SHA256,
        PBKDF2,
        getRandomBytes,
        getLocalStorage,
        createTransport,
      });

      this.client = new MTProto({
        api_id: appID,
        api_hash: appHash,
        test: true,
      });
    }
    // this.client.setDefaultDc(2);
  }

  async call(method, params, options, hideLog = false) {
    try {
      !hideLog && console.log(`${method} req\n${params}`);
      options = {
        ...options,
      };
      let resp = await this.client.call(method, params, options);
      !hideLog && console.dir(`${method} resp\n${resp}`);
      return resp;
    } catch (error) {
      !hideLog && console.log(`${method} error:`, error);

      const { error_code, error_message } = error;

      if (error_code === 303) {
        const [, dcIdAsString] = error_message.split("_MIGRATE_");

        const dcId = Number(dcIdAsString);

        await this.client.setDefaultDc(dcId);

        return this.call(method, params, options);
      }
      return Promise.reject(error);
    }
  }
}

async function uploadBigFile(bytes, callback) {
  let file_id = rand_id();
  console.log("file_id", file_id);

  let uploadBytes = new Uint8Array(bytes);
  console.log("uploading file", bytes);
  const file_total_parts = Math.ceil(uploadBytes.length / 524288);
  console.log("file_total_parts", file_total_parts);
  for (let i = 0; i < file_total_parts; i++) {
    console.log("push part: ", i);
    try {
      let tempBytes = uploadBytes.slice(i * 524288, (i + 1) * 524288);
      if (i === file_total_parts - 1) {
        tempBytes = uploadBytes.slice(i * 524288);
      }
      console.log("tempBytes", tempBytes);
      let finished = await Telegram.call("upload.saveBigFilePart", {
        file_id: file_id,
        file_part: i,
        file_total_parts: file_total_parts,
        bytes: tempBytes,
      });
      console.log("finished", finished);
      if (callback) {
        callback(((i + 1) / file_total_parts) * 100);
      }
    } catch (error) {
      return Promise.reject(error);
    }
  }

  console.log("uploaded");
  return { file_id: file_id, total_part: file_total_parts };
}

function inputFile(file_id, part_number, file_name) {
  return {
    _: "inputFileBig",
    id: file_id,
    parts: part_number,
    name: file_name,
  };
}

function sendFile(inputFile, type) {
  let random_id = rand_id();
  console.log("random_id", random_id);
  return Telegram.call("messages.sendMedia", {
    peer: {
      _: "inputPeerSelf",
    },
    media: {
      _: "inputMediaUploadedDocument",
      file: inputFile,
      mime_type: type,
      attributes: [
        {
          _: "documentAttributeFilename",
          file_name: inputFile.name,
        },
      ],
    },
    random_id: rand_id(),
    message: "test",
  });
}

async function downloadFile(fileDocument) {
  let partSize = 1024 * 1024;
  const file_total_parts = Math.ceil(fileDocument.size / partSize);

  let results = [];
  for (let i = 0; i < file_total_parts; i++) {
    try {
      let result = await Telegram.call("upload.getFile", {
        location: {
          _: "inputDocumentFileLocation",
          id: fileDocument.id,
          access_hash: fileDocument.access_hash,
          file_reference: fileDocument.file_reference,
          thumb_size: "",
        },
        limit: partSize,
        offset: partSize * i,
      });
      console.log("result", result);
      results.push(result);
    } catch (error) {
      return Promise.reject(error);
    }
  }

  console.log("messages.getDocument:", results);
  let bytes = new Uint8Array(fileDocument.size);
  for (let i = 0; i < results.length; i++) {
    for (let j = 0; j < results[i].bytes.length; j++) {
      bytes[i * partSize + j] = results[i].bytes[j];
    }
  }
  let downloadBytes = bytes;
  console.log("downloadBytes", downloadBytes);
  let blob = new Blob([downloadBytes], {
    type: fileDocument.mime_type,
  });
  let url = window.URL.createObjectURL(blob);
  let a = document.createElement("a");
  a.href = url;
  a.download = fileDocument.file_name;
  a.click();
  return Promise.resolve(true);
}

async function sendSignInCode(phone) {
  try {
    let result = await Telegram.call("auth.sendCode", {
      phone_number: phone,
      settings: {
        _: "codeSettings",
      },
    });
    return Promise.resolve(result.phone_code_hash);
  } catch (error) {
    return Promise.reject(error);
  }
}

async function signIn(phone, code, phone_code_hash) {
  try {
    let result = await Telegram.call("auth.signIn", {
      phone_number: phone,
      phone_code_hash: phone_code_hash,
      phone_code: code,
    });
    return Promise.resolve(result);
  } catch (error) {
    return Promise.reject(error);
  }
}

function between(min, max) {
  return Math.floor(Math.random() * (max - min) + min);
}

function rand_id() {
  return between(100000, 999999);
}

const appID = 18987971;
const appHash = "fcfd9e6ed3f9e48a360bb57cc0d59d98";
let Telegram = new TelegramHelper(appID, appHash);

export default Telegram;
export {
  uploadBigFile,
  inputFile,
  sendFile,
  downloadFile,
  sendSignInCode,
  signIn,
};
